import React from 'react';
import { Settings as LayoutSettings, PageLoading,BasicLayoutProps } from '@ant-design/pro-layout';
import {message, notification, Spin} from 'antd';
import qs from 'qs';
import { history, RequestConfig, RunTimeLayoutConfig, ErrorShowType } from 'umi';
import RightContent from '@/components/RightContent';
import Footer from '@/components/Footer';
import { ResponseError } from 'umi-request';
import { MenuDataItem } from '@umijs/route-utils';
import { LoadingOutlined } from '@ant-design/icons';
import {
  queryCurrent,
  selectNavList
} from './services/user';
import defaultSettings from '../config/defaultSettings';
import { Icon } from '@ant-design/compatible';

const antIcon = <LoadingOutlined style={{ fontSize: 24 }} spin />;

/**
 * 获取用户信息比较慢的时候会展示一个 loading
 */
export const initialStateConfig = {
  loading: <PageLoading/>,
};

Spin.setDefaultIndicator(antIcon);// 设置默认加载图标

export async function getInitialState(): Promise<{
  settings?: LayoutSettings;
  currentUser?: API.CurrentUser;
  globalConfig?: {},
  fetchUserInfo?: () => Promise<API.CurrentUser | undefined>;
  fetchNavList?: () => Promise<API.CurrentUser | undefined>;
  getGlobalConfig?: () => Promise<API.CurrentUser | undefined>;
  menuData: MenuDataItem[];
}> {
  const fetchNavList = async () => {
    const currentNavList = await selectNavList();
    if(currentNavList.error.length === 0){
      return currentNavList.data || [];
    }else{
      return [];
    }
  };
  const fetchUserInfo = async () => {
    try {
      const currentUser = await queryCurrent();
      currentUser.data.token = currentUser.token;
      return currentUser.data;
    } catch (error) {
      history.push('/user/login');
    }
    return undefined;
  };
  // 系统的相关配置
  //      window.globalConfig = {};
  // 			globalConfig.context = "";
  // 			globalConfig.portalHost = "\/\/sf.jishutao.com\/portal\/2.0.6";
  // 			globalConfig.staticDomain = "\/\/static.jishutao.com";
  // 			globalConfig.staticHost = "\/\/static.jishutao.com\/1.1.61";
  // 			globalConfig.uploadPath = null;
  // 			globalConfig.uploadPrivatePath = null;
  // 			globalConfig.avatarHost = "\/\/static.jishutao.com";
  // TODO 这里要加个接口返回
  const getGlobalConfig = async () => {
    return {
      avatarHost: '//static.jishutao.com',
      adminData: {
        mobile : "admin",
        name : "\u79D1\u5FB7\u9879\u76EE\u7BA1\u7406\u7CFB\u7EDF",
        email : "2304089483@qq.com",
        createTime : "2018-11-01T11:13:18.000+08:00",
        uid : "1",
        departmentId : "04d7a497-f009-41fc-af89-538e4f05e9b2",
        isSuperAdmin : true,
        isApprovalDecision : false
      }
    }
  };
  // 如果是登录页面，不执行
  if (history.location.pathname !== '/user/login') {
    const currentUser = await fetchUserInfo();
    const currentNavList = currentUser && await fetchNavList();
    const globalConfig = await getGlobalConfig();
    return {
      fetchUserInfo,
      fetchNavList,
      currentUser,
      globalConfig,
      getGlobalConfig,
      menuData: currentNavList,
      settings: defaultSettings,
    };
  }
  return {
    menuData: [],
    fetchUserInfo,
    fetchNavList,
    getGlobalConfig,
    settings: defaultSettings,
  };
}

// 处理后端返回的菜单数据
const handleMenuList = (menuList)=>{
  menuList.map((value)=>{
    value.path = value.url;
    setMenuIcon(value);
    if(value.subMenus){
      value.routes = value.subMenus;
      handleMenuList(value.routes);
    }
    if(value.list){
      value.routes = value.list;
      handleMenuList(value.routes);
    }
    return value
  })
  return menuList;
}

// 设置导航图标
const setMenuIcon = (value)=>{
  switch (value.url) {
    case 'admin' : value.icon = <Icon type="control" />;break;
    case 'personnel' : value.icon = <Icon type="team" />;break;
  }
}

export const layout: RunTimeLayoutConfig = ({ initialState }: {
  initialState: {
    settings?: LayoutSettings;
    menuData: Promise<BasicLayoutProps>;
    currentUser?: API.CurrentUser;
  };
}) => {
  return {
    rightContentRender: () => <RightContent />,
    disableContentMargin: false,
    footerRender: () => <Footer />,
    onPageChange: () => {
      const { location } = history;
      // 如果没有登录，重定向到 login
      if (!initialState?.currentUser && location.pathname !== '/user/login') {
        history.push('/user/login');
      }
    },
    menuDataRender: (menuData) => {
      return initialState && initialState.menuData && Array.isArray(initialState.menuData) ? handleMenuList(initialState.menuData) : menuData;
    },
    menuHeaderRender: undefined,
    // 自定义 403 页面
    // unAccessible: <div>unAccessible</div>,
    ...initialState?.settings,
  };
};

/**
 * 异常处理程序
 * {
 *  "name":"BizError",        //name 为BizError是因为success为false
 *  "data":{
 *    "error":[{"field":"","message":"账号或密码不匹配。"}],
 *    "token":"90c77e0f-cd3b-4555-89bf-21b69dc8b454"
 *  },
 *  "info":{
 *    "success":false,
 *    "data":null,
 *    "errorCode":"",
 *    "errorMessage":"账号或密码不匹配。",
 *    "showType":2,
 *    "traceId":"-1",
 *    "host":"-1"
 *  }
 * }
 */
const errorHandler = (error: ResponseError) => {
  const { info } = error;
  if (error.name === 'BizError') {
    if(info.errorCode === '403'){
      message.warning('登录超时请重新登录');
      history.push('/user/login');
    }else{
      message.warning(info.errorMessage);
    }
  }
  if (!info) {
    notification.error({
      description: '您的网络发生异常，无法连接服务器,请尝试刷新页面',
      message: '网络异常',
    });
  }
  throw info;
};

export const request: RequestConfig = {
  errorHandler,
  errorConfig: {
    adaptor: (resData,ctx)=>{
      return {
        success: !(resData.error && resData.error[0] && resData.error[0].field === '403'),// 只有没登录才是返回false
          data: resData.error || null,
        errorCode: (resData.error && resData.error.length > 0) ? resData.error[0].field : 0,
        errorMessage: (resData.error && resData.error.length > 0) ? resData.error[0].message : '',
        showType: (resData.error && resData.error.length > 0) ? (resData.error[0].field === '403' ? ErrorShowType.REDIRECT : ErrorShowType.ERROR_MESSAGE) : 0,
        traceId: ctx.res ? ctx.res.token : '',
        host: ctx.req ? ctx.req.originUrl : '-1',
      };
    },
    errorPage: '/user/login',       // showType为9是跳这个地址
  },
  requestInterceptors: [
    (url, config) => {
      if (config.method === 'post') {
        config.data = qs.stringify({
          ...(config.data || {}),
        }, {arrayFormat: 'brackets'});
      }
      config.headers = {
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8;',
        'X-Requested-With' : 'XMLHttpRequest',
      }
      return {
        url: `${url}`,
        options: { ...config, interceptors: true },
      };
    },
  ],
  middlewares: [
    async function middlewareA(ctx, next) {
      await next();
    }
  ],
  responseInterceptors: [
    async (config) => {
      return config;
    }
  ]
};
